@@ -20,7 +20,8 @@ convertNodeSetToR(xmlNodeSetPtr obj, SEXP fun, int encoding, SEXP manageMemory)
     SETCAR(expr, fun);
     arg = CDR(expr);
   } else if(TYPEOF(fun) == LANGSXP) {
-    expr = fun;
+    // change from Tomas Kalibera 2016-11-10
+    PROTECT(expr = duplicate(fun));
     arg = CDR(expr);
   }
 
@@ -33,7 +34,7 @@ convertNodeSetToR(xmlNodeSetPtr obj, SEXP fun, int encoding, SEXP manageMemory)
 	  SET_NAMES(ref, mkString(el->name));
 #else
 	  PROTECT(ref = ScalarString(mkCharCE((el->children && el->children->content) ? XMLCHAR_TO_CHAR(el->children->content) : "", encoding)));
-	  SET_NAMES(ref, ScalarString(mkCharCE(el->name, encoding)));
+	  SET_NAMES(ref, ScalarString(mkCharCE((const char *)el->name, encoding)));
 #endif
 	  SET_CLASS(ref, mkString("XMLAttributeValue"));
 	  UNPROTECT(1);
@@ -52,10 +53,10 @@ convertNodeSetToR(xmlNodeSetPtr obj, SEXP fun, int encoding, SEXP manageMemory)
       SET_VECTOR_ELT(ans, i, ref);
   }
 
-  if(expr) {
-    if(TYPEOF(fun) == CLOSXP || TYPEOF(fun) == BUILTINSXP)
-      UNPROTECT(1);
-  } else
+  // change from Tomas Kalibera 2016-11-10
+  if(expr)
+    UNPROTECT(1);
+  else
     SET_CLASS(ans, mkString("XMLNodeSet"));
 
   UNPROTECT(1);
@@ -158,7 +159,6 @@ R_addXMLInternalDocument_finalizer(SEXP sdoc, SEXP fun)
 #if R_XML_DEBUG_WEAK_REFS
     LastDoc = sdoc;
 #endif
-    void *ptr = R_ExternalPtrAddr(sdoc);
     if(TYPEOF(fun) == CLOSXP) {
 	R_RegisterFinalizer(sdoc, fun);	
 	return(sdoc);
@@ -171,6 +171,7 @@ R_addXMLInternalDocument_finalizer(SEXP sdoc, SEXP fun)
 
     R_RegisterCFinalizer(sdoc, action);
 #ifdef R_XML_DEBUG_WEAK_REFS
+    void *ptr = R_ExternalPtrAddr(sdoc);
     int status = R_findExtPtrWeakRef(ptr);
     fprintf(stderr, "is weak ref %d\n", status);
 #endif
